/**
 * Created by aaronlin on 2017/8/10.
 */
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import ReactNative, {
    View,
    Text,
    requireNativeComponent,
    UIManager,
    ViewPropTypes,
    ColorPropType,
} from 'react-native';


const NativePicker = requireNativeComponent('AndroidWheelView', PickerView);
/**
 *
 */
export default class PickerView extends PureComponent{

    static propTypes = {
        ...ViewPropTypes,
        data:PropTypes.array,     //数据源  字符串数组
        itemTextColor:ColorPropType,    //字体颜色
        selectedItemTextColor:ColorPropType,    //被选中的字体颜色
        itemTextSize:PropTypes.number,    //字体大小
        itemSpace:PropTypes.number, //数据项之间间距
        selectedValue:PropTypes.number,    //默认选中的位置, 只能是数字   默认：0
        onValueChange:PropTypes.func, //某一项被选中时执行此回调
        tag:PropTypes.object,
    }

    static defaultProps = {
        selectedValue:0,    //
    }

    constructor(props){
        super(props);
    }

    /**
     * 正在滚动
     * @param e
     * @private
     */
    _onWheelScrolled = (e) => {
        let { offset } = e.nativeEvent;
        //console.log('_onWheelScrolled===============> ', offset);
    }

    /**
     * 滚动停止回调
     * @param e
     * @private
     */
    _onWheelSelected = (e) => {
        let { position } = e.nativeEvent;
        //console.log('_onWheelSelected===============> ', position);
    }

    /**
     * 滚动状态变化   state： 0：停止   1： 被触摸 DRAGGING   2：正在滚动
     * @param e
     * @private
     */
    _onWheelScrollStateChanged = (e) => {
        let { state } = e.nativeEvent;
        //console.log('_onWheelScrollStateChanged===============> ', state);
    }

    /**
     * 被选中回调
     * @param e
     * @private
     */
    _onItemSelected = (e) => {
        let { data, position } = e.nativeEvent;
        //console.log('_onItemSelected===============> ', data, '    ', position);

        this.props.onValueChange && this.props.onValueChange(data, position, this.props.tag);
    }

    render(){

        let { data = [], selectedValue } = this.props;

        return (
            <NativePicker
                ref='nativePicker'
                {...this.props}
                data={JSON.stringify(data)}
                selectedPosition={selectedValue}
                onWheelScrolled={this._onWheelScrolled}
                onWheelSelected={this._onWheelSelected}
                onWheelScrollStateChanged={this._onWheelScrollStateChanged}
                onItemSelected={this._onItemSelected}
            />
        );
    }

    /**
     * 滚动到指定的 position
     * @param position
     * @param anim  是否有动画
     */
    smoothScrollToPosition = (position = 0, anim = true)=>{
        let pickerHandle = this._getPickerHandle();
        if(pickerHandle){
            UIManager.dispatchViewManagerCommand(pickerHandle, UIManager.AndroidWheelView.Commands.smoothScrollToPosition, [position, anim]);
        }
    }

    /**
     *
     * @private
     */
    _getPickerHandle = ()=>{
        if(this.refs.nativePicker){
            return ReactNative.findNodeHandle(this.refs.nativePicker);
        }
    }

    /**
     *
     * @param props
     */
    setNativeProps = (props)=>{
        this.refs.nativePicker && this.refs.nativePicker.setNativeProps(props);
    }

    setData=(dataList)=>{
        console.log("setData=======>",dataList);
        this.setNativeProps({data:JSON.stringify(dataList)});
    }

    setSelectIndex=(index)=>{
        this.setNativeProps({selectedPosition: index});
    }
}