/**
 * Created with IntelliJ IDEA.
 * User: aaronlin
 * Date: 2018/5/9
 * Time: 10:36
 */

import React,{ PureComponent } from 'react';

import PropTypes from 'prop-types';

import {
    View,
    Animated,
    StyleSheet,
} from 'react-native';

import WMRadioGroup from '../WMRadioGroup';

import {
    StyleSheetPropType,
    ViewStylePropTypes,
    TextStylePropTypes,
} from '../RNExt';

const viewStylePropType = StyleSheetPropType(ViewStylePropTypes);
const textStylePropType = StyleSheetPropType(TextStylePropTypes);

/**
 * 动画时间
 * @type {number}
 */
const ANIM_DURATION = 200;
/**
 * 滑块宽度
 * @type {number}
 */
const SLIDE_WIDTH = 13;

const styles = {
    container:{
        height:'100%',
        alignSelf:'center',
        backgroundColor:'#ffffff',
    },
    normalTxtStyle:{
        fontSize:16,
        color:'#333333',
    },
    checkedTxtStyle:{
        fontSize:16,
        color:'#0075FF',
    },
    slide:{
        width:SLIDE_WIDTH,
        height:3,
        borderRadius:2,
        backgroundColor:'#0075FF',
    }
}

/**
 * 滑块菜单
 */
export default class SlideMenuView extends PureComponent{

    static propTypes = {
        ...WMRadioGroup.propTypes,
        /**
         * 菜单布局样式
         */
        menuContainerStyle:viewStylePropType,
        /**
         * 滑块样式
         */
        slideStyle:viewStylePropType,
        /**
         * 菜单点击回调
         */
        onPress:PropTypes.func,
    }

    static defaultProps = {
        data:[],
        menuContainerStyle:{
            height:'92%',
            alignSelf:'center'
        },
        slideStyle:styles.slide,
        normal:{
            icon:null,
            txtStyle:styles.normalTxtStyle
        },
        checked:{
            icon:null,
            txtStyle:styles.checkedTxtStyle
        },
        defaultIndex:0,
        space:50,
    }

    /**
     * 菜单文本宽度
     * @type {number}
     * @private
     */
    _menuTxtWidth = 0;

    /**
     * 滑块宽度
     * @type {number}
     * @private
     */
    _slideWidth = SLIDE_WIDTH;

    constructor(props) {
        super(props);

        this._slideWidth = this._getSlideWidth();

        this.state = {
            slideTranslateXAnim: new Animated.Value(0),
        }
    }

    componentWillUnmount() {
        this._endSlideAnim();
    }

    /**
     * 获取滑块宽度
     */
    _getSlideWidth = ()=>{
        let { slideStyle: propsSlideStyle } = this.props;
        let slideWidth;

        if(propsSlideStyle){
            if(Array.isArray(propsSlideStyle)){
                for(let index = 0; index<propsSlideStyle.length; index++){
                    let pStyle = propsSlideStyle[index];
                    if(typeof pStyle === 'number'){
                        pStyle = StyleSheet.flatten(pStyle);
                    }
                    if(pStyle.width){
                        slideWidth = pStyle.width;
                    }
                }
            }else{
                if(typeof propsSlideStyle === 'number'){
                    propsSlideStyle = StyleSheet.flatten(propsSlideStyle);
                }
                if(propsSlideStyle.width){
                    slideWidth = propsSlideStyle.width;
                }
            }
            if(typeof slideWidth !== 'number'){
                throw new Error('滑块宽度请设置数字');
            }
        }
        return slideWidth || SLIDE_WIDTH
    }

    /**
     * 菜单点击
     * @param e
     * @param radioIndex
     * @param itemData
     * @private
     */
    _menuPress = (e, radioIndex, itemData)=>{
        this._startSlideAnim(radioIndex, true);

        this.props.onPress && this.props.onPress(e, radioIndex, itemData);
    }

    /**
     *
     * @param e
     * @param radioIndex
     * @private
     */
    _radioItemLayout = (e, radioIndex)=>{
        const { defaultIndex, space, data } = this.props;
        const { nativeEvent } = e;

        if(radioIndex === 0){
            if(data.length === 1){
                this._menuTxtWidth = nativeEvent.layout.width;
            }else{
                this._menuTxtWidth = nativeEvent.layout.width - space;
            }
            this._startSlideAnim(defaultIndex, false);
        }
    }

    /**
     * 开始滑块动画
     * @param isAnim
     * @private
     */
    _startSlideAnim = (targetIndex : number = 0, isAnim : boolean = true)=>{
        const { space, data } = this.props;

        let toTranslateX = 0;
        if(targetIndex === 0){
            toTranslateX = Math.abs(this._menuTxtWidth - this._slideWidth) / 2;
        }else{
            toTranslateX = (targetIndex + 1) * this._menuTxtWidth + targetIndex * space - this._menuTxtWidth / 2 - this._slideWidth / 2;
        }

        if(isAnim){
            Animated.timing(this.state.slideTranslateXAnim, {
                toValue:toTranslateX,
                duration: ANIM_DURATION,
                isInteraction: false,
                useNativeDriver: true,
            }).start();
        }else{
            this.state.slideTranslateXAnim.setValue(toTranslateX);
        }
    }

    /**
     * 停止滑块动画
     * @private
     */
    _endSlideAnim = ()=>{
        this.state.slideTranslateXAnim.stopAnimation();
    }

    /**
     * 获取选中的数据
     */
    get checkedData(){
        return this.refs.radioGroup && this.refs.radioGroup.checkedData;
    }

    /**
     * 获取选中的index
     * @returns {Number|*}
     */
    get checkedIndex(){
        return this.refs.radioGroup && this.refs.radioGroup.checkedIndex;
    }

    /**
     * 设置被选中项
     * @param index
     */
    setCheck(index : number, isPerformPress : boolean = true){
        this.refs.radioGroup && this.refs.radioGroup.setCheck(index, isPerformPress);
        if(!isPerformPress){
            this._startSlideAnim(index, true);
        }
    }

    /**
     *
     * @param props
     */
    setNativeProps = (props)=>{
        this.refs.containerView && this.refs.containerView.setNativeProps(props);
    }

    render(){
        const { style, menuContainerStyle, slideStyle, onPress, onLayout, data, ...radioGroupProps } = this.props;

        if(data.length === 0){
            return null;
        }

        return (
            <View ref={'containerView'} style={[styles.container, style]} onLayout={onLayout} pointerEvents={'box-none'}>

                <WMRadioGroup
                    {...radioGroupProps}
                    data={data}
                    ref={'radioGroup'}
                    style={[menuContainerStyle]}
                    onItemLayout={this._radioItemLayout}
                    onPress={this._menuPress}
                />

                <Animated.View style={[slideStyle, {transform:[{translateX:this.state.slideTranslateXAnim}]}]} pointerEvents={'none'}/>
            </View>
        );
    }

}